{% load i18n admin_urls %}
<div class="submit-row">
    {% block submit-row %}
        {% if show_save %}
            <input type="submit"
                   value="{% translate 'Save' %}"
                   class="default"
                   name="_save">
        {% endif %}
    </div>
