# Generated by Django 5.0.7 on 2024-08-28 02:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_chunk_file_upload", "0002_alter_filemanager_options_and_more"),
    ]

    operations = [
        migrations.RenameIndex(
            model_name="filemanager",
            new_name="filemanager_checksum_idx",
            old_name="file_manager_checksum_idx",
        ),
    ]
